; MOON8 NES project
; Brad Smith, 2012
;
; bank_play.s
;
; Music player code.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; common data

; track text centred (32 tiles wide)
text_song_title:
    .byte "           Speak to Me          "
    .byte "             Breathe            "
    .byte "           On the Run           "
    .byte "              Time              "
    .byte "    The Great Gig in the Sky    "
    .byte "              Money             "
    .byte "           Us and Them          "
    .byte "       Any Colour You Like      "
    .byte "          Brain Damage          "
    .byte "             Eclipse            "
;          012345678901234**789012345678901

; PPU update buffer for player visualizer

ROW_BARS0 = $2000 +  9 + (15 * 32) ; 15 bytes, VU meter bars per channel
ROW_BARS1 = $2000 +  9 + (16 * 32) ; 15 bytes, second row of VU
ROW_WAVES = $2000 +  9 + (18 * 32) ; 15 bytes, 
ROW_TIME  = $2000 + 12 + (24 * 32) ; 8 bytes: timer and single/continue indicator

ppu_buffer_play:
    ;
    .byte 15, >ROW_BARS0, <ROW_BARS0
    ;.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    .byte "AA BB CC DD EE "
    ;
    .byte 15, >ROW_BARS1, <ROW_BARS1
    ;.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    .byte "AA BB CC DD EE "
    ;
    .byte 15, >ROW_WAVES, <ROW_WAVES
    .byte "  ", 0
    .byte "  ", 0
    .byte TILE_TRIANGLE+0, TILE_TRIANGLE+1, 0
    .byte "  ", 0
    .byte TILE_DPCM+0, TILE_DPCM+1, 0
    ;.byte "AA BB CC DD EE "
    ;
    .byte  8, >ROW_TIME , <ROW_TIME
    .byte "00:00", 0, 0, 0
    ;.byte "00:00 --"
    ;
    .byte 0

PPU_BUFFER_BARS0 = 3
PPU_BUFFER_BARS1 = PPU_BUFFER_BARS0 + 15 + 3
PPU_BUFFER_WAVES = PPU_BUFFER_BARS1 + 15 + 3
PPU_BUFFER_TIME  = PPU_BUFFER_WAVES + 15 + 3
PPU_BUFFER_LEN   = PPU_BUFFER_TIME  + 8  + 1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;

.include "ft_driver.s"

play_init:
; 
;   ready to play tune
;
    LDY zp_song
    LDX song_table_index, Y
    ;   index = 0
    BNE :+
    ZP_ADDR music0, ft_music_addr
    JMP @play_begin
    : ; index = 1
    DEX
    BNE :+
    ZP_ADDR music1, ft_music_addr
    JMP @play_begin
    : ; index = 2
    DEX
    BNE :+
    ZP_ADDR music2, ft_music_addr
    JMP @play_begin
    : ; index = 3
    ZP_ADDR music3, ft_music_addr
@play_begin:
    STB $0F, $4015 ; recover from pause if necessary
    STB 0, zp_play_done
    STB 0, zp_play_tune
    RTS
    
play_load_song:
;
;   alternative to Famitracker::ft_music_init which would reset APU registers
;   and other variables which would put a temporary break in sound.
;   A = song index in FTM
;   X = 0-NTSC, 1-PAL
;
    ZP_FREQ0 = zp_base + 0
    ZP_FREQ1 = zp_base + 1
    ; temporarily store high freq to prevent pop
    PHA
    LDA var_ch_PrevFreqHigh + 0
    STA ZP_FREQ0
    LDA var_ch_PrevFreqHigh + 1
    STA ZP_FREQ1
    PLA
    ;
    ASL
    JSR Famitracker::ft_load_song
    ; restore high freq
    LDA ZP_FREQ0
    STA var_ch_PrevFreqHigh + 0
    LDA ZP_FREQ1
    STA var_ch_PrevFreqHigh + 1
    ; ready
    RTS

play_tick:
;
;   call once per frame
;
    LDA zp_play_tune
    CMP #PLAY_NONE
    BEQ @play_none
    CMP #PLAY_CONTINUE
    BEQ @play_continue
    ; new song selected in A, initialize it
    LDX zp_pal
    JSR play_load_song
    STB PLAY_CONTINUE, zp_play_tune
@play_continue:
    ; tick the playing track
    JSR Famitracker::ft_music_play
    JSR play_check_done
    ; update timer
    LDA zp_play_done
    BNE @timer_ticked
    INC zp_time + 0
        LDA zp_time + 0
        CMP zp_fps
        BCC @timer_ticked
        STB 0, zp_time + 0
    INC zp_time + 1
        LDA zp_time + 1
        CMP #10
        BCC @timer_ticked
        STB 0, zp_time + 1
    INC zp_time + 2
        LDA zp_time + 2
        CMP #6
        BCC @timer_ticked
        STB 0, zp_time + 2
    INC zp_time + 3
        LDA zp_time + 3
        CMP #10
        BCC @timer_ticked
        STB 0, zp_time + 3
    INC zp_time + 4
        LDA zp_time + 4
        CMP #6
        BCC @timer_ticked
        STB 0, zp_time + 4 ; roll over after 1 hour
@timer_ticked:        
@play_none:
    RTS

play_pause:
    STB %00000000, $4015 ; silence APU, clears length counteres
    RTS

play_unpause:
    STB %00001111, $4015 ; restore APU
    ; restore square 0
    LDA var_ch_PeriodCalcHi + 0
    STA $4003
    ; restore square 1
    LDA var_ch_PeriodCalcHi + 1
    STA $4007
    ; triangle is restored by famitracker driver every frame
    ; noise is restored by famitracker every frame
    ; DPCM is transient, will be restored by next play
    RTS

play_check_done:
    LDA var_PlayerFlags ; =0 when C00 is reached
    BEQ :+
        RTS
    :
    LDA zp_single
    BNE @stop_play
        LDA zp_song
        CMP #9
        BEQ @stop_play ; if song 9 don't switch
        INC zp_song
        STB 3, zp_switch
        ; NOTE: the very first frame of the row C00 is on will play before the switch
    RTS
    @stop_play:
        ZP_ADDR silent, ft_music_addr
        STB 0, zp_play_tune
        STB 1, zp_play_done
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ppu routines

ppu_off:
    LDA #0
    STA PPU_MASK ; disable render
    STA PPU_CTRL ; disable NMI
    RTS

ppu_on:
    LDA #%10000000 ; enable NMI
    STA PPU_CTRL
    RTS

ppu_update:
;
; called by NMI to update nametable in vblank
;
; bss_ppu = 256 byte PPU update stream
; data is in stream of form:
;   A B C [...]
;   A = number of bytes of data (A = 0 means no more data)
;   B, C = high, low PPU address to begin write
;   [...] = A bytes of data
; Be careful not to attempt more than 256 bytes of data (counter will loop)
;
    LDA PPU_STAT ; reset latch
    LDY #0
@packet_loop:
    LDA bss_ppu, Y
    BEQ @load_finished
    TAX
    INY
    LDA bss_ppu, Y
    STA PPU_ADDR
    INY
    LDA bss_ppu, Y
    STA PPU_ADDR
    INY
@fill_loop:
    LDA bss_ppu, Y
    STA PPU_DATA
    INY
    DEX
    BNE @fill_loop
    JMP @packet_loop
@load_finished:
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; controller read

; store three controller reads temporarily
ZP_GAMEPAD_TEMP = zp_base

gamepad_read:
;
;   safely reads first gamepad state
;
    ; read controller 3 times to avoid DPCM bus conflict
    .repeat 3, I
        ; strobe to latch data
        LDA #1
        STA GAMEPAD0
        LDA #0
        STA GAMEPAD0
        ; store bit to test when all bits have been read
        LDA #%10000000
        STA ZP_GAMEPAD_TEMP + I
        ;
    :
        LDA GAMEPAD0
        ROR
        ROR ZP_GAMEPAD_TEMP + I
        BCC :-
    .endrepeat
    ; at least two reads should be the same, put the 2 of 3 result in A
    LDA ZP_GAMEPAD_TEMP + 0
    CMP ZP_GAMEPAD_TEMP + 1
    BEQ @gamepad_return ; A = 0 + 1
    CMP ZP_GAMEPAD_TEMP + 2
    BEQ @gamepad_return ; A = 0 + 2 (1 is different)
    LDA ZP_GAMEPAD_TEMP + 1
    ;                   ; A = 1 (presume 1 = 2)
@gamepad_return:
    RTS
    
gamepad_pressed:
;
;   updates zero page gamepad variables:
;     zp_gamepad       = current state
;     zp_gamepad_last  = last frame
;     zp_gamepad_press = new pressed buttons this frame
;
    ; remember previous frame
    LDA zp_gamepad
    STA zp_gamepad_last
    ; read current frame
    JSR gamepad_read
    STA zp_gamepad
    ; determine new presses
    LDA zp_gamepad_last
    EOR #%11111111 ; A = gamepad not pressed last frame
    AND zp_gamepad ; A = gamepad pressed this frame
    STA zp_gamepad_press
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; player routines

ZP_TRACK_NAME = zp_base + 0

draw_track_title:
    ; load track
    ; add song * 32 to text_song_title
    LDA #<text_song_title
    STA ZP_TRACK_NAME + 0
    LDA #>text_song_title
    STA ZP_TRACK_NAME + 1
    LDA zp_song
    ASL ; song * 32
    ASL
    ASL
    ASL
    ASL
    PHA ; add the high bit to the load address (song >= 8)
    BCC @no_high_bit
    INC ZP_TRACK_NAME + 1
@no_high_bit:
    PLA
    CLC
    ADC ZP_TRACK_NAME + 0
    STA ZP_TRACK_NAME + 0
    LDA #0
    ADC ZP_TRACK_NAME + 1
    STA ZP_TRACK_NAME + 1
    ; store 32 bytes from ZP_TRACK_NAME to bss_ppu at pixel 0,22
    STB 32, bss_ppu+0
    TRACK_NAME_PPU_ADDR = $2000 + 0 + (22 * 32)
    STB >TRACK_NAME_PPU_ADDR, bss_ppu+1
    STB <TRACK_NAME_PPU_ADDR, bss_ppu+2
    LDY #0
@name_loop:
    LDA (ZP_TRACK_NAME), Y
    STA bss_ppu+3, Y
    INY
    CPY #32
    BNE @name_loop
    STB 0, bss_ppu+3+32
    RTS


draw_song_animation:
;
;   renders music animation data to bss_ppu
;
    ZP_TEMP = zp_base + 0 ; temp variable
    ;
    ; draw timer
    ;
    LDA zp_time + 4
        CLC
        ADC #'0'
        CMP #'0'
        BNE :+
        LDA #' ' ; hide leading 0
    :
        STA bss_ppu + PPU_BUFFER_TIME + 0 ; minutes 10s
    LDA zp_time + 3
        CLC
        ADC #'0'
        STA bss_ppu + PPU_BUFFER_TIME + 1 ; minutes 1s
    LDA zp_play_tune
        CMP #PLAY_NONE
        BNE :+
            LDA #TILE_PAUSE
            JMP :++
        :
            LDA #':'
        :
        STA bss_ppu + PPU_BUFFER_TIME + 2
    LDA zp_time + 2
        CLC
        ADC #'0'
        STA bss_ppu + PPU_BUFFER_TIME + 3 ; seconds 10s
    LDA zp_time + 1
        CLC
        ADC #'0'
        STA bss_ppu + PPU_BUFFER_TIME + 4 ; seconds 1s
    ; single/continuous
    LDA zp_single
        ASL
        CLC
        ADC #TILE_SINGLE
        STA bss_ppu + PPU_BUFFER_TIME + 6
        ADC #1
        STA bss_ppu + PPU_BUFFER_TIME + 7
    ;
    ; VU meters
    ;
    ; square 0
    LDA var_ch_Note + 0
        BNE :+ ; note is off
            LDA #0
            JMP :++
        : ;      note is on
            LDA var_ch_VolColumn + 0
            ASL
            AND #$F0
            ORA var_ch_OutVolume + 0
            TAX
            LDA Famitracker::ft_volume_table, X
        :
        CMP #8
        BCS :+
            ; volume < 8
            CLC
            ADC #TILE_VU0
            STA bss_ppu + PPU_BUFFER_BARS1 + 0
            STA bss_ppu + PPU_BUFFER_BARS1 + 1
            LDA #0
            STA bss_ppu + PPU_BUFFER_BARS0 + 0
            STA bss_ppu + PPU_BUFFER_BARS0 + 1
            JMP :++
        :
            ; volume > 8
            CLC
            ADC #(TILE_VU0 - 8)
            STA bss_ppu + PPU_BUFFER_BARS0 + 0
            STA bss_ppu + PPU_BUFFER_BARS0 + 1
            LDA #(TILE_VU0 + 7)
            STA bss_ppu + PPU_BUFFER_BARS1 + 0
            STA bss_ppu + PPU_BUFFER_BARS1 + 1
        :
    ; square 1
    LDA var_ch_Note + 1
        BNE :+ ; note is off
            LDA #0
            JMP :++
        : ;      note is on
            LDA var_ch_VolColumn + 1
            ASL
            AND #$F0
            ORA var_ch_OutVolume + 1
            TAX
            LDA Famitracker::ft_volume_table, X
        :
        CMP #8
        BCS :+
            ; volume < 8
            CLC
            ADC #TILE_VU2
            STA bss_ppu + PPU_BUFFER_BARS1 + 3
            STA bss_ppu + PPU_BUFFER_BARS1 + 4
            LDA #0
            STA bss_ppu + PPU_BUFFER_BARS0 + 3
            STA bss_ppu + PPU_BUFFER_BARS0 + 4
            JMP :++
        :
            ; volume > 8
            CLC
            ADC #(TILE_VU2 - 8)
            STA bss_ppu + PPU_BUFFER_BARS0 + 3
            STA bss_ppu + PPU_BUFFER_BARS0 + 4
            LDA #(TILE_VU2 + 7)
            STA bss_ppu + PPU_BUFFER_BARS1 + 3
            STA bss_ppu + PPU_BUFFER_BARS1 + 4
        :
    ; triangle
    LDA var_ch_Volume + 2
        BEQ :+
        LDA var_ch_Note + 2
        BEQ :+
            ; on
            LDA #(TILE_VU0 + 7)
            STA bss_ppu + PPU_BUFFER_BARS0 + 6
            STA bss_ppu + PPU_BUFFER_BARS0 + 7
            STA bss_ppu + PPU_BUFFER_BARS1 + 6
            STA bss_ppu + PPU_BUFFER_BARS1 + 7
            JMP :++
        :
            ; off
            LDA #TILE_VU0
            STA bss_ppu + PPU_BUFFER_BARS1 + 6
            STA bss_ppu + PPU_BUFFER_BARS1 + 7
            LDA #0
            STA bss_ppu + PPU_BUFFER_BARS0 + 6
            STA bss_ppu + PPU_BUFFER_BARS0 + 7
        :
    ; noise
    LDA var_ch_Note + 3
        BNE :+ ; note is off
            LDA #0
            JMP :++
        : ;      note is on
            LDA var_ch_VolColumn + 3
            ASL
            AND #$F0
            ORA var_ch_OutVolume + 3
            TAX
            LDA Famitracker::ft_volume_table, X
        :
        STA ZP_TEMP ; store for reuse in waveform animation below
        CMP #8
        BCS :+
            ; volume < 8
            CLC
            ADC #TILE_VU1
            STA bss_ppu + PPU_BUFFER_BARS1 + 9
            STA bss_ppu + PPU_BUFFER_BARS1 + 10
            LDA #0
            STA bss_ppu + PPU_BUFFER_BARS0 + 9
            STA bss_ppu + PPU_BUFFER_BARS0 + 10
            JMP :++
        :
            ; volume > 8
            CLC
            ADC #(TILE_VU1 - 8)
            STA bss_ppu + PPU_BUFFER_BARS0 + 9
            STA bss_ppu + PPU_BUFFER_BARS0 + 10
            LDA #(TILE_VU1 + 7)
            STA bss_ppu + PPU_BUFFER_BARS1 + 9
            STA bss_ppu + PPU_BUFFER_BARS1 + 10
        :
    ; DPCM
    LDA zp_dpcm_anim
        CMP #8
        BCS :+
            ; volume < 8
            CLC
            ADC #TILE_VU2
            STA bss_ppu + PPU_BUFFER_BARS1 + 12
            STA bss_ppu + PPU_BUFFER_BARS1 + 13
            LDA #0
            STA bss_ppu + PPU_BUFFER_BARS0 + 12
            STA bss_ppu + PPU_BUFFER_BARS0 + 13
            JMP :++
        :
            ; volume > 8
            CLC
            ADC #(TILE_VU2 - 8)
            STA bss_ppu + PPU_BUFFER_BARS0 + 12
            STA bss_ppu + PPU_BUFFER_BARS0 + 13
            LDA #(TILE_VU2 + 7)
            STA bss_ppu + PPU_BUFFER_BARS1 + 12
            STA bss_ppu + PPU_BUFFER_BARS1 + 13
        :
        ; fade to 0
        LDA zp_dpcm_anim
        BEQ :+
            DEC zp_dpcm_anim
        :
    ;
    ; waveforms
    ;
    LDA var_ch_DutyCycle + 0 ; square 0
        ASL
        CLC
        ADC #TILE_DUTY
        STA bss_ppu + PPU_BUFFER_WAVES + 0
        ADC #1
        STA bss_ppu + PPU_BUFFER_WAVES + 1
    LDA var_ch_DutyCycle + 1 ; square 1
        ASL
        CLC
        ADC #TILE_DUTY
        STA bss_ppu + PPU_BUFFER_WAVES + 3
        ADC #1
        STA bss_ppu + PPU_BUFFER_WAVES + 4
    LDA ZP_TEMP ; noise volume (calculated above)
        BEQ :++ ; channel is silent if ZP_TEMP == 0
            ; speed of noise anim depends on current noise frequency
            INC zp_noise_anim + 1 ; increment sub-anim
            LDA var_ch_PeriodCalcLo + 3 ; noise
            AND #$0F
            EOR #$0F
            CMP zp_noise_anim + 1 ; if noise freq <= sub_anim, do tick
            BCS :+
                INC zp_noise_anim + 0 ; tick noise anim
                STB 0, zp_noise_anim + 1
            :
        :
        LDA zp_noise_anim + 0 ; clamp sprite anim 0-7
        AND #$07
        STA zp_noise_anim + 0
        CLC
        ADC #TILE_NOISE
        STA bss_ppu + PPU_BUFFER_WAVES + 9
        LDA zp_noise_anim + 0
        CLC
        ADC #1
        AND #$07
        CLC
        ADC #TILE_NOISE
        STA bss_ppu + PPU_BUFFER_WAVES + 10
    ;
    ; done updating draw list
    ;
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main_code

ZP_PLAY_TITLE = zp_base + 0

bank_init:
    JSR draw_track_title
    STB 1, zp_frame_ready
    ; setup music
    JSR play_init
    ; turn on rendering
    JSR ppu_on
    ; finish frame
    LDA zp_frame
@wait_1_nmi:
    CMP zp_frame
    BEQ @wait_1_nmi
    ; fill bss_ppu with template
    LDX #0
@ppu_update_fill:
    LDA ppu_buffer_play, X
    STA bss_ppu, X
    INX
    CPX #PPU_BUFFER_LEN
    BNE @ppu_update_fill
    JMP play_begin_loop
    ;
    ; player loop
    ;
play_loop:
    LDA zp_frame
@wait_nmi:
    CMP zp_frame
    BEQ @wait_nmi
play_begin_loop:
    ; read gamepad
    JSR gamepad_pressed
    ; play sound
    LDA zp_play_mode
    ;
        ; slow play, plays only every other frame
        CMP #PLAY_SLOW
        BNE :+
        LDA zp_frame
        AND #$01
        BNE @play_ticked
        JSR play_tick
        JMP @play_ticked
    :
        ; fast play, plays eight ticks
        CMP #PLAY_FAST
        BNE :+
        JSR play_tick
        .repeat 7
            LDA zp_switch ; don't keep ticking if tune needs to be switched
            BNE @play_ticked
            JSR play_tick
        .endrepeat
        JMP @play_ticked
    :
    ; regular play, just one tick
    JSR play_tick
@play_ticked:
    ; animate
    JSR draw_song_animation
    ; handle input
    ;
        LDA zp_gamepad_press
        AND #(GP_A)
        BEQ :+
        STB 0, zp_single ; continuous play
        LDA zp_play_done ; restart if done
        BEQ :+
        STB 2, zp_switch
    :
        LDA zp_gamepad_press
        AND #(GP_B)
        BEQ :+
        STB 1, zp_single ; single play
        LDA zp_play_done ; restart if done
        BEQ :+
        STB 2, zp_switch
    :
        LDA zp_gamepad_press
        AND #(GP_START)
        BEQ :++
        LDA zp_play_tune
        CMP #PLAY_NONE
        BEQ :+
        JSR play_pause
        STB PLAY_NONE, zp_play_tune ; pause
        JMP :++
        :
        JSR play_unpause
        STB PLAY_CONTINUE, zp_play_tune ; unpause
    :
        LDA zp_gamepad_press
        AND #(GP_SELECT)
        BEQ :+
        STB 1, zp_switch
    :
        LDA zp_gamepad_press
        AND #(GP_U)
        BEQ :+
        LDA zp_song
        BEQ :+ ; if song 0 don't switch
        DEC zp_song
        STB 2, zp_switch
    :
        LDA zp_gamepad_press
        AND #(GP_D)
        BEQ :+
        LDA zp_song
        CMP #9
        BEQ :+ ; if song 9 don't switch
        INC zp_song
        STB 2, zp_switch
    :
    ; slow/fast playback goes by button held, not button press
    STB PLAY_NORMAL, zp_play_mode ; reset slow/fast flags
    ;
        LDA zp_gamepad
        AND #(GP_L)
        BEQ :+
        STB PLAY_SLOW, zp_play_mode
    :
        LDA zp_gamepad
        AND #(GP_R)
        BEQ :+
        STB PLAY_FAST, zp_play_mode
    :
    ; if zp_switch is flaggged, return to bank 0 (main menu)
    LDA zp_switch
    BEQ @no_switch
    CMP #1 ; zp_switch: 1 main menu, 2 swap, 3 continue to next
    BEQ @main_menu
    ;
    ; song swap
    ;
    CMP #2
    BNE :+
        ; reset timer
        LDA #0
        STA zp_time + 0
        STA zp_time + 1
        STA zp_time + 2
        STA zp_time + 3
        STA zp_time + 4
    :
    ; swap to appropriate song bank
    STB 0, zp_switch
    LDX zp_song
    LDY song_table_bank, X
    JMP bank_swap
    ;
    ; return to main menu
    ;
@main_menu:
    STB 0, zp_switch
    JSR ppu_off
    STB TUNE_SILENT, zp_play_tune
    LDY #0
    JMP bank_swap
@no_switch:
    ; mark frame ready
    STB 1, zp_frame_ready
    ; next frame
    .if PROFILE
        LDA #%00011111
        STA PPU_MASK
    .endif
    JMP play_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NMI rendering routine

bank_nmi:
    PHA
    TXA
    PHA
    TYA
    PHA
    ; if new data for PPU, load it
    LDA zp_frame_ready
    BEQ @done_ppu_update
    ; upload sprite OAM via DMA
    LDA #0
    STA OAM_ADDR
    LDA #>bss_oam
    STA OAM_DMA
    ; update ppu memory if requested
    JSR ppu_update
    STB 0, zp_frame_ready
    ; enable render
    LDA #%00011110
    STA PPU_MASK
@done_ppu_update:
    LDA PPU_STAT
    ; reset scroll
    LDA #0
    STA PPU_SCRL
    STA PPU_SCRL
    INC zp_frame
    PLA
    TAY
    PLA
    TAX
    PLA
    RTI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; IRQ (not used)

bank_irq:
    RTI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NSF playback

.ifdef NSF

nsf_bank_init:
    JSR play_init
    LDX zp_pal
    JSR play_load_song
    LDA zp_song
    BNE end_nsf_bank_init
        ; song zero starts with more than 3s silence, trim it
        ; to bypass silence detection
        ZP_COUNT = zp_base + 0
        STB 120, ZP_COUNT
        :
            JSR Famitracker::ft_music_play
            DEC ZP_COUNT
        BNE :-
end_nsf_bank_init:
    RTS

nsf_bank_play:
    JSR Famitracker::ft_music_play
    LDA zp_play_done ; already stopped
    BEQ :+
        RTS
    :
    LDA var_PlayerFlags ; =0 when C00 is reached
    BEQ :+
        RTS
    :
    LDA zp_single
    BNE @stop_play
        LDA zp_song
        CMP #4
        BEQ @stop_play ; end of side 1
        CMP #9
        BEQ @stop_play ; end of side 2
        INC zp_song
        LDA zp_song
        TAX
        LDA song_table_bank, X
        JMP nsf_bank_swap
        ; NOTE: the very first frame of the row C00 is on will play before the switch
    @stop_play:
        ZP_ADDR silent, ft_music_addr
        LDA #0
        LDX zp_pal
        JSR Famitracker::ft_music_init
        STB 1, zp_play_done
    RTS

.endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; end of file
