; MOON8 NES project
; Brad Smith, 2012
;
; moon8.s
;
; Global ROM compilation unit. This file .includes all
; other source code, and arranges the ROM.
;
; Note that every bank includes bank_stub.s and bank_vect.s,
; which reference 'bank_nmi', 'bank_init', and 'bank_irq' code
; labels that must be defined in each bank.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; iNES header

.ifndef NSF
.ifndef PRG

.segment "HEADER"

; NES header
.byte 'N', 'E', 'S', $1A ; ID
.byte 16 ; 16k PRG bank count (16 = 256kb)
.byte $00 ; 8k CHR bank count (0 = RAM)
.byte $70, $00 ; mapper ($07 = AxROM)
.byte $0, $0, $0, $0, $0, $0, $0, $0

.endif
.endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; global defines like RAM usage

.include "global.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Bank 0 contains init code and main menu

.scope PRG0
.segment "PRG0_8000"
    .include "bank_main.s"
    .include "ft_vol_mul.s" ; multiplicative volume
.segment "PRG0_STUB"
    .include "bank_stub.s"
.segment "PRG0_VECT"
    .include "bank_vect.s"
.endscope

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Banks 1-7 contain music data and identical player code

; song_table_bank and song_table_index in bank_stub.s
; point to these banks and music# labels

.scope PRG1
.segment "PRG1_8000"
    music0:
        .incbin "music/0_speak_to_me.bin"
    music1:
        .incbin "music/2_on_the_run.bin"
    music2:
    music3:
    silent:
        .incbin "music/silent.bin"
.segment "PRG1_C000"
    dpcm:
        .assert * = $C000, error, "DPCM not at $C000"
        .incbin "music/0_dpcm.bin"
    .include "bank_play.s"
    .include "ft_vol_lin.s" ; linear volume
.segment "PRG1_STUB"
    .include "bank_stub.s"
.segment "PRG1_VECT"
    .include "bank_vect.s"
.endscope

.scope PRG2
.segment "PRG2_8000"
    music0:
        .incbin "music/1_breathe.bin"
    music1:
        .incbin "music/4_sky.bin"
    music2:
    music3:
    silent:
        .incbin "music/silent.bin"
.segment "PRG2_C000"
    dpcm:
        .assert * = $C000, error, "DPCM not at $C000"
        .incbin "music/1_dpcm.bin"
        ; same: "music/4_dpcm.bin"
    .include "bank_play.s"
    .include "ft_vol_lin.s" ; linear volume
.segment "PRG2_STUB"
    .include "bank_stub.s"
.segment "PRG2_VECT"
    .include "bank_vect.s"
.endscope

.scope PRG3
.segment "PRG3_8000"
    music0:
        .incbin "music/3_time.bin"
    music1:
    music2:
    music3:
    silent:
        .incbin "music/silent.bin"
.segment "PRG3_C000"
    dpcm:
        .assert * = $C000, error, "DPCM not at $C000"
        .incbin "music/3_dpcm.bin"
    .include "bank_play.s"
    .include "ft_vol_lin.s" ; linear volume
.segment "PRG3_STUB"
    .include "bank_stub.s"
.segment "PRG3_VECT"
    .include "bank_vect.s"
.endscope

.scope PRG4
.segment "PRG4_8000"
    music0:
        .incbin "music/5_money.bin"
    music1:
    music2:
    music3:
    silent:
        .incbin "music/silent.bin"
.segment "PRG4_C000"
    dpcm:
        .assert * = $C000, error, "DPCM not at $C000"
        .incbin "music/5_dpcm.bin"
    .include "bank_play.s"
    .include "ft_vol_mul.s" ; multiplicative volume
.segment "PRG4_STUB"
    .include "bank_stub.s"
.segment "PRG4_VECT"
    .include "bank_vect.s"
.endscope

.scope PRG5
.segment "PRG5_8000"
    music0:
        .incbin "music/6_us_and_them.bin"
    music1:
    music2:
    music3:
    silent:
        .incbin "music/silent.bin"
.segment "PRG5_C000"
    dpcm:
        .assert * = $C000, error, "DPCM not at $C000"
        .incbin "music/6_dpcm.bin"
    .include "bank_play.s"
    .include "ft_vol_mul.s" ; multiplicative volume
.segment "PRG5_STUB"
    .include "bank_stub.s"
.segment "PRG5_VECT"
    .include "bank_vect.s"
.endscope

.scope PRG6
.segment "PRG6_8000"
    music0:
        .incbin "music/7_any_colour.bin"
    music1:
        .incbin "music/8_brain.bin"
    music2:
    music3:
    silent:
        .incbin "music/silent.bin"
.segment "PRG6_C000"
    dpcm:
        .assert * = $C000, error, "DPCM not at $C000"
        .incbin "music/7_dpcm.bin"
        ; same: "music/8_dpcm.bin"
    .include "bank_play.s"
    .include "ft_vol_mul.s" ; multiplicative volume
.segment "PRG6_STUB"
    .include "bank_stub.s"
.segment "PRG6_VECT"
    .include "bank_vect.s"
.endscope

.scope PRG7
.segment "PRG7_8000"
    music0:
        .incbin "music/9_eclipse.bin"
    music1:
    music2:
    music3:
    silent:
        .incbin "music/silent.bin"
.segment "PRG7_C000"
    dpcm:
        .assert * = $C000, error, "DPCM not at $C000"
        .incbin "music/9_dpcm.bin"
    .include "bank_play.s"
    .include "ft_vol_mul.s" ; multiplicative volume
.segment "PRG7_STUB"
    .include "bank_stub.s"
.segment "PRG7_VECT"
    .include "bank_vect.s"
.endscope

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NSF header

.ifdef NSF

.segment "HEADER"

; NSF header
.byte 'N', 'E', 'S', 'M', $1A ; ID
.byte $01 ; version
.byte 12 ; songs
.byte 1 ; starting song
.word $8000 ; load address
.word PRG0::nsf_init ; init function
.word PRG0::nsf_play ; play function
; title
.byte "MOON8",  0,0,0,0,0,0,0,0,0,0,0
.byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
; artist
.byte "bradsmith / Pink Floyd"
.byte             0,0,0,0,0,0,0,0,0,0
; copyright
.byte "2010 / 1973 - rainwarrior.ca"
.byte                         0,0,0,0


; other
.word 16666 ; NTSC speed
.byte 0,1,2,3,4,5,6,7 ; bankswitch init
.word 20000 ; PAL speed
.byte %00000010 ; PAL/NTSC bits
.byte %00000000 ; expansion bits
.byte 0,0,0,0 ; pad to $80

.endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; end of file
